/**
 * Date:            Time:       User:               Description:
 * 07.05.2006       17:57       Smart Max           creation of the file DemoBuilder.java
 *
 */
package org.max;

import static org.max.NativeSystem.*;

import org.max.xml.index.XMLBuilder;
import org.max.xml.index.IndexFabric;
import org.max.xml.index.IndexFabricSerializer;

import java.util.List;
import java.util.ArrayList;
import java.io.IOException;

import net.sf.xanswer.indexing.IndexException;

/**
 * DemoBuilder
 */
public class DemoBuilder {
    private static void error(String message) {
        System.out.println("Error. " + message);
        System.exit(1);
    }

    private static void printAbout() {
        System.out.println("Demo Index Fabric builder");
        System.out.println("Written by Maxim Podkolzine, 2006, Borland Academy");
        System.out.println("Usage:");
        System.out.println("DemoBuilder file1 [file2 ... fileN] -d directory");
        System.out.println("\nExample of usage:");
        System.out.println("DemoBuilder file1.xml \"xml\\file1.xml\" \"xml\\file2.xml\" -d demo");
    }

    public static void main(String[] args) throws IndexException {
        if (args.length == 0) {
            printAbout();
            return;
        }

        if (args.length < 3) {
            error("Not enough actual parameters");
        }

        boolean dir = false;
        List<String> files = new ArrayList<String>();
        String directory = null;

        for (String s : args) {
            if (s.equals("-d")) {
                if (dir) {
                    error("Incorrect input syntax");
                }

                dir = true;
            } else {
                if (dir) {
                    if (directory != null) {
                        error("Incorrect input syntax");
                    }

                    directory = s;
                } else {
                    files.add(s);
                }
            }
        }

        if (!exists(directory)) {
            if (!createDirectory(directory)) {
                error("Failed to created directory \"" + directory + "\"");
            }
        }

        XMLBuilder builder = new XMLBuilder();
        for (String name : files) {
            try {
                builder.addXMLDocument(name, newInputStream(name));
                System.out.println("File \"" + name + "\" successfully added");
            } catch (IOException e) {
                System.out.println("File \"" + name + "\" not found");
            }
        }

        IndexFabric indexFabric = builder.getIndexFabric();
        IndexFabricSerializer serializer = new IndexFabricSerializer(indexFabric, directory);
        serializer.writeFabric();

        System.out.println("Index Fabric was successfully created and serialized to the destination directory");
    }
}
