/**
 * Date:            Time:       User:               Description:
 * 07.05.2006       18:29       Smart Max           creation of the file DemoRetriever.java
 *
 */
package org.max;

import static org.max.NativeSystem.*;

import org.max.xml.index.XMLRetriever;
import org.max.xml.index.IndexFabricDeserializer;
import org.max.xml.index.IndexFabric;
import org.max.xml.index.XMLReference;
import org.max.trie.INode;
import org.max.trie.IReference;

import net.sf.xanswer.indexing.IndexException;

import java.util.List;
import java.util.ArrayList;

/**
 * DemoRetriever
 */
public class DemoRetriever {
    private static void error(String message) {
        System.out.println("Error. " + message);
        System.exit(1);
    }

    private static void printAbout() {
        System.out.println("Demo Index Fabric retriever");
        System.out.println("Written by Maxim Podkolzine, 2006, Borland Academy");
        System.out.println("Usage:");
        System.out.println("DemoRetriever directory tag1 [tag2 ... fileN]");
        System.out.println("\nExample of usage:");
        System.out.println("DemoRetriever demo project component entry");
    }

    public static void main(String[] args) throws IndexException {
        if (args.length == 0) {
            printAbout();
            return;
        }

        if (args.length < 2) {
            error("Not enough actual parameters");
        }

        String directory = args[0];

        if (!exists(directory)) {
            error("Failed to open directory \"" + directory + "\"");
        }

        List<String> tags = new ArrayList<String>();
        for (int i = 1; i < args.length; i++) {
            tags.add(args[i]);
        }

        IndexFabricDeserializer deserializer = new IndexFabricDeserializer();
        IndexFabric indexFabric = deserializer.readIndexFabric(directory);

        System.out.println("Index Fabric successfully opened");

        XMLRetriever retriever = new XMLRetriever(indexFabric);
        INode node = retriever.retrieveRawPath(tags);

        if (node == null) {
            System.out.println("The key was not found in Index Fabric");
        } else {
            System.out.print("The key was successfully found in Index Fabric");
            if (node.getReferences() != null) {
                System.out.println("\nThis path is presented in the follwing documents:");

                for (IReference reference : node.getReferences()) {
                    XMLReference xmlReference = (XMLReference) reference;
                    System.out.println("doc ID = \"" + xmlReference.getDocID() + "\", " +
                                       "tag number = " + xmlReference.getTagIndex());
                }
            } else {
                System.out.println(": the found node has no references on documents");
            }
        }
    }
}
