/**
 * Date:            Time:       User:               Description:
 * 04.05.2006       23:26       Smart Max           creation of the file NativeSystem.java
 *
 */
package org.max;

import java.io.*;

/**
 * NativeSystem
 */
public abstract class NativeSystem {
    /**
     * Holds a system-dependent name-separator.
     * <p>
     * For example, it can be slash ('<tt>/</tt>') or backslash (<tt>'\'</tt>).
     */
    public static final String Separator = File.separator;

    /**
     * Creates a new input stream by location name
     *
     * @param   name                                the location name
     * @return                                      new input stream
     * @throws  IOException                         if I/O errors occur
     */
    public static InputStream newInputStream(String name) throws IOException {
        return new FileInputStream(name);
    }

    /**
     * Creates a new output stream by location name
     *
     * @param   name                                the location name
     * @return                                      new output stream
     * @throws  IOException                         if I/O errors occur
     */
    public static OutputStream newOutputStream(String name) throws IOException {
        return new FileOutputStream(name);
    }

    /**
     * Creates a new reader, given the name of location to read from
     *
     * @param   name                                the location name
     * @return                                      the reader from specified location
     * @throws  IOException                         if I/O errors occur
     */
    public static Reader newReader(String name) throws IOException {
        return new FileReader(name);
    }

    /**
     * Creates a new writer, given the name of location to write to
     *
     * @param   name                                the location name
     * @return                                      the writer to specified location
     * @throws  IOException                         if I/O errors occur
     */
    public static Writer newWriter(String name) throws IOException {
        return new FileWriter(name);
    }

    /**
     * Returns if specified location name exists
     *
     * @param   name                                the location name
     * @return                                      true iff such location exists
     */
    public static boolean exists(String name) {
        return new File(name).exists();
    }

    /**
     * Creates a new directory in system, given the name.
     * <p>
     * Returns the result of operation.
     *
     * @param   name                                the location name
     * @return                                      true iff creation succeeded
     */
    public static boolean createDirectory(String name) {
        return new File(name).mkdir();
    }
}
