/**
 * Date:            Time:       User:               Description:
 * 11.05.2006       11:55       Smart Max           creation of the file NodeTest.java
 *
 */
package org.max.trie.impl;

import static org.max.NativeSystem.*;

import junit.framework.TestCase;

import org.max.xml.index.XMLBuilder;
import org.max.trie.INode;
import org.max.trie.ISymbol;
import org.max.trie.IPatriciaTrie;

import java.io.*;
import java.util.List;

import net.sf.xanswer.indexing.IndexException;

/**
 * NodeTest
 */
public class NodeTest extends TestCase {
    private static final String testDirectory = "test" + Separator;

    public void test_SerializeInt() throws IOException {
        ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "int.dat"));
        output.writeInt(0);
        output.close();
    }

    private static class TestSymbol implements Serializable, ISymbol {
        private static final long serialVersionUID = 111111111111111111L;
        String name = "value";
        int type = 100;
    }

    private static class TestShortSymbol implements Serializable, ISymbol {
        private static final long serialVersionUID = 22222222222222222L;
        short id = -100;
    }

    public void test_SerializeIntNullSymbol1() throws IOException {
        ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "rootNode (refined).dat"));
        output.writeInt(0);
        output.writeObject(null);
        output.writeObject(new TestSymbol());
        output.close();
    }

    public void test_SerializeIntNullSymbol2() throws IOException {
        ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "rootNode (refined, short).dat"));
        output.writeInt(0);
        output.writeObject(null);
        output.writeObject(new TestShortSymbol());
        output.close();
    }

    public void test_SerializeRootNode() throws IOException {
        Node n = new Node();
        ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "rootNode.dat"));
        output.writeObject(n);
        output.close();
    }

    public void test_SerializeInternalNode() throws IOException, IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        INode root = xmlBuilder.getIndexFabric().getTrie().getRoot();

        ISymbol label = root.getOutcomingSymbols().iterator().next();
        INode n = root.getChild(label);

        ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "internalNode.dat"));
        output.writeObject(n);
        output.close();
    }

    public void test_SerializeBlocksOf1() throws IOException, IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        xmlBuilder.addXMLDocument("xml" + Separator + "doc2.xml");
        xmlBuilder.addXMLDocument("xml" + Separator + "project.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        for (int i = 1; i < 100; i++) {
            PatriciaTrieDivider divider = new PatriciaTrieDivider();
            PatriciaTrieDivider.setBlockSize(i);

            List<Block> list = divider.divide(trie, "");
            Block b = null;
            for (Block block : list) {
                b = block;
                if (b.size() == i) {
                    break;
                } else {
                    b = null;
                }
            }

            if (b != null) {
                System.out.println("b" + i + ".dat -> " + b.size());
                ObjectOutput output = new ObjectOutputStream(newOutputStream(testDirectory + "b" + i + ".dat"));
                output.writeObject(b);
                output.close();
            }
        }

    }
}