/**
 * Date:            Time:       User:               Description:
 * 20.04.2006       11:50       Smart Max           creation of the file PatriciaTrieDividerTest.java
 *
 */
package org.max.trie.impl;

import static org.max.NativeSystem.*;

import junit.framework.TestCase;
import org.max.xml.index.XMLBuilder;
import org.max.trie.IPatriciaTrie;

import java.util.Collection;

import net.sf.xanswer.indexing.IndexException;

/**
 * PatriciaTrieDividerTest
 */
public class PatriciaTrieDividerTest extends TestCase {
    // Tests for doc1.xml decompositions

    public void test_Simple() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        PatriciaTrieDivider divider = new PatriciaTrieDivider();
        PatriciaTrieDivider.setBlockSize(5);
        Collection<Block> blocks = divider.divide(trie, "test_Simple_Dir");

        //System.out.println("Trie size: " + trie.size());
        int sum = 0;
        for (Block block : blocks) {
            int size = block.size();
            assertTrue(size > 0);
            assertTrue(size <= PatriciaTrieDivider.getBlockSize());
            sum += size;

            //System.out.println("    block size = " + size);
        }

        assertEquals(trie.size(), sum);
    }

    private void checkTrie(String xmlName, int blockSize) throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument(xmlName);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        PatriciaTrieDivider divider = new PatriciaTrieDivider();
        PatriciaTrieDivider.setBlockSize(blockSize);
        Collection<Block> blocks = divider.divide(trie, "checkTrie" + xmlName + blockSize);

        {   // Here we divide the trie using Divider1 and compare result blocks number
            xmlBuilder = new XMLBuilder();
            xmlBuilder.addXMLDocument(xmlName);
            trie = xmlBuilder.getIndexFabric().getTrie();

            TestDivider1 testDivider1 = new TestDivider1();
            TestDivider1.setBlockSize(blockSize);
            Collection<Block> blocks1 = testDivider1.divide(trie, "");
            assertTrue(blocks.size() <= blocks1.size());
            //System.out.println(blocks.size() + " <= " + blocks1.size());
        }

        // We count average block size: trie.size() / blocks.size()
        // if average block size < blockSize / 2, then test outputs warning (but not fails)
        if (blockSize * blocks.size() > 2 * trie.size()) {
            System.out.println("Warning!");
            System.out.println("average block size = " + 1.0 * trie.size() / blocks.size());
            System.out.println("maximal block size = " + blockSize);
        }

        //System.out.println("Check for: " + xmlName + " with max size = " + blockSize);
        //System.out.println("Trie size: " + trie.size());
        //System.out.println("Total blocks number: " + blocks.size());
        int sum = 0;
        for (Block block : blocks) {
            int size = block.size();
            assertTrue(size > 0);
            assertTrue(size <= PatriciaTrieDivider.getBlockSize());
            sum += size;

            //System.out.println("    block size = " + size);
        }

        assertEquals(trie.size(), sum);
    }

    public void test_SampleDoc1_1() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 1);
    }

    public void test_SampleDoc1_2() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 2);
    }

    public void test_SampleDoc1_3() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 3);
    }

    public void test_SampleDoc1_4() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 4);
    }

    public void test_SampleDoc1_5() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 5);
    }

    public void test_SampleDoc1_6() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 6);
    }

    public void test_SampleDoc1_7() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 7);
    }

    public void test_SampleDoc1_8() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 8);
    }

    public void test_SampleDoc1_9() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 9);
    }

    public void test_SampleDoc1_10() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 10);
    }

    public void test_SampleDoc1_15() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 15);
    }

    public void test_SampleDoc1_24() throws IndexException {
        checkTrie("xml" + Separator + "doc1.xml", 24);
    }

    // Tests for doc2.xml decompositions

    public void test_SampleDoc2_1() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 1);
    }

    public void test_SampleDoc2_2() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 2);
    }

    public void test_SampleDoc2_3() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 3);
    }

    public void test_SampleDoc2_4() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 4);
    }

    public void test_SampleDoc2_5() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 5);
    }

    public void test_SampleDoc2_6() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 6);
    }

    public void test_SampleDoc2_7() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 7);
    }

    public void test_SampleDoc2_8() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 8);
    }

    public void test_SampleDoc2_9() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 9);
    }

    public void test_SampleDoc2_10() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 10);
    }

    public void test_SampleDoc2_15() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 15);
    }

    public void test_SampleDoc2_23() throws IndexException {
        checkTrie("xml" + Separator + "doc2.xml", 23);
    }

    // Tests for project.xml decompositions

    public void test_Project_5() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 5);
    }

    public void test_Project_10() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 10);
    }

    public void test_Project_15() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 15);
    }

    public void test_Project_20() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 20);
    }

    public void test_Project_30() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 30);
    }

    public void test_Project_50() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 50);
    }

    public void test_Project_100() throws IndexException {
        checkTrie("xml" + Separator + "project.xml", 100);
    }

    // Tests for web.xml decompositions

    public void test_Web_10() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 10);
    }

    public void test_Web_20() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 20);
    }

    public void test_Web_30() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 30);
    }

    public void test_Web_40() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 40);
    }

    public void test_Web_50() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 50);
    }

    public void test_Web_100() throws IndexException {
        checkTrie("xml" + Separator + "web.xml", 100);
    }

    // Tests for unicode.xml

    public void test_Unicode_10() throws IndexException {
        checkTrie("xml" + Separator + "unicode.xml", 10);
    }

    public void test_Unicode_50() throws IndexException {
        checkTrie("xml" + Separator + "unicode.xml", 50);
    }

    // Tests for mscorlib.xml decompositions

    public void test_Mscorlib_10() throws IndexException {
        checkTrie("xml" + Separator + "mscorlib.xml", 10);
    }

    public void test_Mscorlib_50() throws IndexException {
        checkTrie("xml" + Separator + "mscorlib.xml", 50);
    }

    public void test_Mscorlib_100() throws IndexException {
        checkTrie("xml" + Separator + "mscorlib.xml", 100);
    }
}
