/**
 * Date:            Time:       User:               Description:
 * 30.04.2006       20:51       Smart Max           creation of the file PatriciaTrieSerializerTest.java
 *
 */
package org.max.trie.impl;

import static org.max.trie.impl.TestUtil.*;
import static org.max.NativeSystem.*;

import junit.framework.TestCase;

import org.max.xml.index.XMLBuilder;
import org.max.trie.IPatriciaTrie;
import org.max.trie.INode;
import org.max.trie.IWritableNode;
import org.max.trie.ISymbol;
import org.max.RandomXML;

import java.io.*;
import java.util.Map;
import java.util.List;

import net.sf.xanswer.indexing.IndexException;

/**
 * PatriciaTrieSerializerTest
 */
public class PatriciaTrieSerializerTest extends TestCase {
    public void test_Simple1() throws IOException, ClassNotFoundException, IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(true);
        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        String location = "test" + Separator + "doc1 [5]";
        PatriciaTrieSerializer serializer = new PatriciaTrieSerializer(trie, location);
        createDirectory(location);
        serializer.writeTrie(5);

        BufferedReader reader = new BufferedReader(newReader(location + Separator + "meta.dat"));
        String rootBlockName = reader.readLine();
        reader.close();

        ObjectInputStream input = new ObjectInputStream(
                newInputStream(location + Separator + rootBlockName));
        Block block = (Block) input.readObject();

        INode root1 = trie.getRoot();
        INode root2 = block.getSubTrie();
        assertEqualSubTries(root1, root2);
    }

    public void test_Simple2() throws IOException, ClassNotFoundException, IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(true);
        xmlBuilder.addXMLDocument("xml" + Separator + "doc2.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        String location = "test" + Separator + "doc2 [5]";
        PatriciaTrieSerializer serializer = new PatriciaTrieSerializer(trie, location);
        createDirectory(location);
        serializer.writeTrie(5);

        BufferedReader reader = new BufferedReader(newReader(location + Separator + "meta.dat"));
        String rootBlockName = reader.readLine();
        reader.close();

        ObjectInputStream input = new ObjectInputStream(newInputStream(location + Separator + rootBlockName));
        Block block = (Block) input.readObject();

        INode root1 = trie.getRoot();
        INode root2 = block.getSubTrie();
        assertEqualSubTries(root1, root2);
    }

    // Tests for project.xml decompositions

    private void checkTrie(String name, IPatriciaTrie trie, int blockSize)
            throws IOException, ClassNotFoundException {
        String location = "test" + Separator + name + " [" + blockSize + "]";
        PatriciaTrieSerializer serializer = new PatriciaTrieSerializer(trie, location);

        createDirectory(location);
        serializer.writeTrie(blockSize);

        BufferedReader reader = new BufferedReader(newReader(location + Separator + "meta.dat"));
        String rootBlockName = reader.readLine();
        reader.close();

        ObjectInputStream input = new ObjectInputStream(newInputStream(location + Separator + rootBlockName));
        Block block = (Block) input.readObject();

        INode root1 = trie.getRoot();
        INode root2 = block.getSubTrie();
        assertEqualSubTries(root1, root2);
    }

    private static IPatriciaTrie getTrie(String ... fileNames) {
        XMLBuilder builder = new XMLBuilder();
        for (String fileName : fileNames) {
            try {
                builder.addXMLDocument("xml" + Separator + fileName);
            } catch (IndexException e) {
                e.printStackTrace();
                fail();
            }
        }
        return builder.getIndexFabric().getTrie();
    }

    private static final IPatriciaTrie ProjectTrie = getTrie("project.xml");

    public void test_Project_10() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 10);
    }

    public void test_Project_20() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 20);
    }

    public void test_Project_30() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 30);
    }

    public void test_Project_40() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 40);
    }

    public void test_Project_50() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 50);
    }

    public void test_Project_100() throws IOException, ClassNotFoundException {
        checkTrie("project", ProjectTrie, 100);
    }

    // Tests for project.xml decompositions

    private static final IPatriciaTrie WebTrie = getTrie("web.xml");

    public void test_Web_10() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 10);
    }

    public void test_Web_20() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 20);
    }

    public void test_Web_30() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 30);
    }

    public void test_Web_40() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 40);
    }

    public void test_Web_50() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 50);
    }

    public void test_Web_100() throws IOException, ClassNotFoundException {
        checkTrie("web", WebTrie, 100);
    }

    // Tests for colors.xml decompositions

    private static final IPatriciaTrie ColorsTrie = getTrie("colors.xml");

    public void test_Colors_30() throws IOException, ClassNotFoundException {
        checkTrie("colors", ColorsTrie, 30);
    }

    public void test_Colors_50() throws IOException, ClassNotFoundException {
        checkTrie("colors", ColorsTrie, 50);
    }

    public void test_Colors_80() throws IOException, ClassNotFoundException {
        checkTrie("colors", ColorsTrie, 80);
    }

    public void test_Colors_250() throws IOException, ClassNotFoundException {
        checkTrie("colors", ColorsTrie, 250);
    }

    // Tests for unicode.xml decompositions

    private static final IPatriciaTrie UnicodeTrie = getTrie("unicode.xml");

    public void test_Unicode_50() throws IOException, ClassNotFoundException {
        checkTrie("unicode", UnicodeTrie, 50);
    }

    public void test_Unicode_100() throws IOException, ClassNotFoundException {
        checkTrie("unicode", UnicodeTrie, 100);
    }

    public void test_Unicode_200() throws IOException, ClassNotFoundException {
        checkTrie("unicode", UnicodeTrie, 200);
    }

    // Tests for System.xml

    private static final IPatriciaTrie SystemTrie = getTrie("System.xml");

    public void test_System_50() throws IOException, ClassNotFoundException {
        checkTrie("System", SystemTrie, 50);
    }

    public void test_System_100() throws IOException, ClassNotFoundException {
        checkTrie("System", SystemTrie, 100);
    }

    // Tests for different files

    private static final IPatriciaTrie ComplexTrie1 = getTrie(
            "attr.xml", "blanks.xml", "books.xml", "doc3.xml", "similarAttrs.xml",
            "similarTags.xml", "tags.xml", "text.xml", "uppercase.xml");

    public void test_Complex1_10() throws IOException, ClassNotFoundException {
        checkTrie("complex-1", ComplexTrie1, 10);
    }

    public void test_Complex1_20() throws IOException, ClassNotFoundException {
        checkTrie("complex-1", ComplexTrie1, 20);
    }

    public void test_Complex1_50() throws IOException, ClassNotFoundException {
        checkTrie("complex-1", ComplexTrie1, 50);
    }

    private static final IPatriciaTrie ComplexTrie2 = getTrie(
            "doc1.xml", "doc2.xml", "doc3.xml", "System.Drawing.xml", "System.Windows.Forms.xml");

    public void test_Complex2_500() throws IOException, ClassNotFoundException {
        checkTrie("complex-2", ComplexTrie2, 500);
    }

    // Tests for random XML files

    private static final RandomXML random = new RandomXML();

    private static IPatriciaTrie getRandomTrie(int tagsNumber, int nameLength) {
        String name = "random" + Separator + System.currentTimeMillis() + ".xml";
        try {
            random.createXML("xml" + Separator + name, tagsNumber, nameLength);
        } catch (IOException e) {
            e.printStackTrace();
            fail();
        }

        return getTrie(name);
    }

    private static final IPatriciaTrie Random1Trie = getRandomTrie(500, 10);

    public void test_Random1_25() throws IOException, ClassNotFoundException {
        checkTrie("random-1", Random1Trie, 25);
    }

    public void test_Random1_50() throws IOException, ClassNotFoundException {
        checkTrie("random-1", Random1Trie, 50);
    }

    private static final IPatriciaTrie Random2Trie = getRandomTrie(10000, 4);

    public void test_Random2_100() throws IOException, ClassNotFoundException {
        checkTrie("random-2", Random2Trie, 100);
    }

    public void test_Random2_200() throws IOException, ClassNotFoundException {
        checkTrie("random-2", Random2Trie, 200);
    }

    private static final IPatriciaTrie Random3Trie = getRandomTrie(20000, 2);

    public void test_Random3_100() throws IOException, ClassNotFoundException {
        checkTrie("random-3", Random3Trie, 100);
    }

    public void test_Random3_200() throws IOException, ClassNotFoundException {
        checkTrie("random-3", Random3Trie, 200);
    }

    // Full serialization

    private static final IPatriciaTrie ComplexTrie3 = getTrie("XRXS1um.xml", "data.xml");

    public void test_FullSerialization1() throws IOException, ClassNotFoundException {
        XMLBuilder.setAddPathPrefixes(false);

        String location = "test" + Separator + "full-1 [100]";
        PatriciaTrieSerializer serializer = new PatriciaTrieSerializer(ComplexTrie3, location);

        createDirectory(location);
        serializer.writeFullTrie(100);

        BufferedReader reader = new BufferedReader(newReader(location + Separator + "meta.dat"));
        String rootBlockName = reader.readLine();
        reader.close();

        ObjectInputStream input = new ObjectInputStream(newInputStream(location + Separator + rootBlockName));
        Block block = (Block) input.readObject();

        INode root1 = ComplexTrie3.getRoot();
        IWritableNode root2 = block.getSubTrie();
        assertEqualSubTries(root1, root2);

        input = new ObjectInputStream(newInputStream(location + Separator + "triemap.dat"));
        Map<IWritableNode, List<ISymbol>> map = (Map<IWritableNode, List<ISymbol>>) input.readObject();

        PatriciaTrie modifiableTrie = new PatriciaTrie(root2, map, true);
        addTrie(modifiableTrie, ComplexTrie1);
    }
}
