/**
 * Date:            Time:       User:               Description:
 * 16.03.2006       13:03       Smart Max           creation of the file XMLBuilderTest.java
 *
 */
package org.max.xml.index;

import static org.max.NativeSystem.*;
import static org.max.xml.index.TestUtil.*;

import junit.framework.TestCase;

import org.max.trie.IPatriciaTrie;
import org.max.trie.ISymbol;
import org.max.util.BidirectionalMap;

import java.io.File;
import java.util.Set;
import java.util.HashSet;

import net.sf.xanswer.indexing.IndexException;

/**
 * XMLBuilderTest
 */
public class XMLBuilderTest extends TestCase {
    public void test_Simple() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.restoreDefaults();

        xmlBuilder.addXMLDocument("xml" + Separator + "doc1.xml");
        xmlBuilder.addXMLDocument("xml" + Separator + "doc2.xml");

        //System.out.println(xmlBuilder);
    }

    public void test_AddSampleDoc1() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "doc1.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkLeafNode(trie.get(PathBuilder.makeList("ibn>ABC Corp")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("iba>1 Industrial Way")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("isn>Acme Inc")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("isa>2 Acme Rd.")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("it>drill")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("itc'>2")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("it>saw")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("itc'>3")), docID);
    }

    public void test_AddSampleDoc2() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "doc2.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkLeafNode(trie.get(PathBuilder.makeList("ibn>Oracle Inc")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("ibp>555-1212")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("isn>IBM Corp")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("itc>4")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("itn>nail")), docID);
    }

    public void test_AddWeb() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "web.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkLeafNode(trie.get(PathBuilder.makeList("wv'>2.4")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wse>default")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("wx'")));
        checkLeafNode(trie.get(PathBuilder.makeList("wsip>debug")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wsip>listings")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wsip>fork")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wsip>xpoweredBy")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wvu>/")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("wvu>*.jsp")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wvu>*.jspx")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wmx>abs")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("wmx>t")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("wmx>tif")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wmx>tiff")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wmx>texinfo")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wm->application/rtf")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("wm->application/xml")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("wm->application/xml-dtd")), docID);
    }

    public void test_AddProject() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "project.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkLeafNode(trie.get(PathBuilder.makeList("pv'>4")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pr'>false")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcn'>AntConfiguration")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcdb'>true")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcn'>CodeStyleSettingsManager")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgityn'>text")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgitya'>Button")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgitya'>RadioButton")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgitya'>CheckBox")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgitya'>Label")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgif-w'>150")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgif-g'>-1")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgif-w'>200")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcgif-g'>200")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcon'>IS_EANABLED")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcon'>ADDITIONAL_OPTIONS_STRING")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcon'>COPY_FORMS_RUNTIME_TO_OUTPUT")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcoa'>true")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("pcoa'>false")), docID);
    }

    public void test_UppercaseInTags() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "uppercase.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkLeafNode(trie.get(PathBuilder.makeList("nN>1")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nA>2")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nm>3")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("na>4")), docID);

        assertNull(trie.get(PathBuilder.makeList("nn")));
        assertNull(trie.get(PathBuilder.makeList("NN")));
    }

    public void test_AttrNames() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "attr.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkInternalNode(trie.get(PathBuilder.makeList("na")));
        checkLeafNode(trie.get(PathBuilder.makeList("nan'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("naa'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nam'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nax'>0")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("nam")));
        checkLeafNode(trie.get(PathBuilder.makeList("namn'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("namy'>1")), docID);
        checkInternalNode(trie.get(PathBuilder.makeList("nat>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nat>name1")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("naz'>2")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nmn'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nma'>name")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nmm'>name1")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nmw'>3")), docID);
        checkLeafNode(trie.get(PathBuilder.makeList("nmt>name2")), docID);
    }

    public void test_SimilarTagNames() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "similarTags.xml");
        BidirectionalMap<String, ISymbol> map = xmlBuilder.getIndexFabric().getTagMap();

        Set<ISymbol> usedSymbols = new HashSet<ISymbol>();
        assertTrue(usedSymbols.add(map.getB("ada")));
        assertTrue(usedSymbols.add(map.getB("a")));
        assertTrue(usedSymbols.add(map.getB("d")));
        assertTrue(usedSymbols.add(map.getB("ad")));
        assertTrue(usedSymbols.add(map.getB("da")));
        assertTrue(usedSymbols.add(map.getB("daa")));
        assertTrue(usedSymbols.add(map.getB("aad")));
        assertTrue(usedSymbols.add(map.getB("aa")));
        assertTrue(usedSymbols.add(map.getB("dd")));
        assertTrue(usedSymbols.add(map.getB("daad")));
        assertTrue(usedSymbols.add(map.getB("adad")));

        assertEquals(usedSymbols.size(), map.size());
    }

    public void test_SimilarAttrNames() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "similarAttrs.xml");
        BidirectionalMap<String, ISymbol> map = xmlBuilder.getIndexFabric().getAttrMap();

        Set<ISymbol> usedSymbols = new HashSet<ISymbol>();
        for (char ch1 = 'a'; ch1 <= 'b'; ch1++) {
            assertTrue(usedSymbols.add(map.getB(String.valueOf(ch1))));

            for (char ch2 = 'a'; ch2 <= 'b'; ch2++) {
                assertTrue(usedSymbols.add(map.getB(String.valueOf(ch1) + ch2)));

                for (char ch3 = 'a'; ch3 <= 'b'; ch3++) {
                    assertTrue(usedSymbols.add(map.getB(String.valueOf(ch1) + ch2 + ch3)));
                }
            }
        }

        assertEquals(usedSymbols.size(), map.size());
    }

    public void test_AddTags() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        String docID = "xml" + Separator + "tags.xml";
        xmlBuilder.addXMLDocument(docID);
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();

        checkInternalNode(trie.get(PathBuilder.makeList("t")));
        checkInternalNode(trie.get(PathBuilder.makeList("ta")));
        //...
    }

    public void test_TextInXML() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        xmlBuilder.addXMLDocument("xml" + Separator + "text.xml");
        BidirectionalMap<String, ISymbol> map = xmlBuilder.getIndexFabric().getTextMap();
        assertTrue(map.size() <= 96);
    }

    public void test_Options_TrueTrue() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(true);
        XMLBuilder.setAddPathPrefixes(true);
        xmlBuilder.addXMLDocument("xml" + Separator + "blanks.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();
        assertEquals(6, trie.size());
    }

    public void test_Options_FlaseTrue() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(false);
        XMLBuilder.setAddPathPrefixes(true);
        xmlBuilder.addXMLDocument("xml" + Separator + "blanks.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();
        assertEquals(11, trie.size());
    }

    public void test_Options_TrueFalse() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(true);
        XMLBuilder.setAddPathPrefixes(false);
        xmlBuilder.addXMLDocument("xml" + Separator + "blanks.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();
        assertEquals(2, trie.size());
    }

    public void test_Options_FalseFalse() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();
        XMLBuilder.setIgnoreWhitespace(false);
        XMLBuilder.setAddPathPrefixes(false);
        xmlBuilder.addXMLDocument("xml" + Separator + "blanks.xml");
        IPatriciaTrie trie = xmlBuilder.getIndexFabric().getTrie();
        assertEquals(8, trie.size());
    }

    public void test_AddAllXML() throws IndexException {
        XMLBuilder xmlBuilder = new XMLBuilder();

        File xmlDir = new File("xml" + Separator);
        String[] fileNames = xmlDir.list();
        for (String name : fileNames) {
            if (!name.equals("random")) {
                xmlBuilder.addXMLDocument(xmlDir.getAbsolutePath() + Separator + name);
            }
        }

        //System.out.println(xmlBuilder.getPatriciaTrie().size());
        //System.out.println(xmlBuilder);
    }
}
